import numpy as np
import csv
import pyper
import sys
import gc
import autoenco2
sys.path.append("./../Util/")

import preparation
import clustering
import evaluation

PARA = sys.argv

csvName = ['ant_v1_7','camel_v1_6','ivy_v1_4','jedit_v4_0','log4j_v1_0','lucene_v2_4','poi_v3_0','tomcat_v6_0','xalan_v2_6','xerces_v1_3']
projectName = "PROMISE"


def data_extend(filename):
    train_data = []
    train_target = []

    f = open(filename,'r')
    dataReader = csv.reader(f)
    for row in dataReader:
        train_data.append(list(map(float,row[3:-1])))
        train_target.append(float(row[-1]))

    f.close()

    train_data = np.array(train_data)

    temp = []
    for i in train_target:
        if i != 0:
            temp.append(1)
        else:
            temp.append(0)

    train_target = np.array(temp)

    return [train_data, train_target]


def cal_10_fold_cv(times1,times2):

    file_name = csvName[int(PARA[3])]

    r = pyper.R()
    r("set.seed(200)")

    r('library("caret")')

    ALL = []

    LMT = [] # Logistic Model Tree
    J48 = [] # Decision Tree (J48)
    LR = [] # Logistic Regression
    NB = [] # Naive Bayes
    RF = [] # Random Forest

    AE_feature_data_list = []

    components = []
    intercepts_visible = []
    for times in range(times1,times2):
    #for times in range(1,2):


        print("+++ {0} times {1} fold +++".format(times,file_name))


        train_data = []
        train_target = []
        test_data = []
        test_target = []

        filename = "./../../data/{0}/{1}/train_times{2}.csv".format(projectName, file_name, times)
        train_data, train_target = data_extend(filename)

        filename = "./../../data/{0}/{1}/test_times{2}.csv".format(projectName, file_name, times)
        test_data, test_target = data_extend(filename)


        train_data_std, test_data_std = preparation.standardization_SVL(train_data,test_data,"0-1scale")


        train_data_std, test_data_std, weight, bias = autoenco2.AE_SVL(train_data_std, test_data_std, EPOCH=200)
        components.append(list(map(list,weight)))
        intercepts_visible.append(bias)

        AE_feature_data_list.append(test_data_std)


        #ans = cslLMTbyR(train_data_std)
        ans_LMT,ans_J48,ans_NB,ans_RF,ans_LR = clustering.Caret(train_data_std,train_target,test_data_std,test_target,r)


        #print('Conventional with LMT')
        temp = evaluation.calAUC_SVL(ans_LMT,test_target)
        #print(temp)
        LMT.append(temp)


        temp = evaluation.calAUC_SVL(ans_J48,test_target)
        #print(temp)
        J48.append(temp)


        temp = evaluation.calAUC_SVL(ans_NB,test_target)
        #print(temp)
        NB.append(temp)


        temp = evaluation.calAUC_SVL(ans_RF,test_target)
        #print(temp)
        RF.append(temp)


        temp = evaluation.calAUC_SVL(ans_LR,test_target)
        #print(temp)
        LR.append(temp)


        del train_data
        del train_target
        del train_data_std
        del test_data
        del test_target
        del test_data_std
        gc.collect()


    ALL.append(LMT)
    ALL.append(J48)
    ALL.append(LR)
    ALL.append(NB)
    ALL.append(RF)



    f = open('./data/{0}/{1}_{2}_{3}.csv'.format(projectName,file_name,times1,times2),'w')
    csvWriter = csv.writer(f)

    for metrics in ALL:
        csvWriter.writerow(metrics)


    f = open('./data/{0}/weight_{1}_{2}_{3}.csv'.format(projectName,file_name,times1,times2),'w')
    csvWriter = csv.writer(f)

    for count,metrics in enumerate(components):
        csvWriter.writerow("".format(count))
        for row in metrics:
            csvWriter.writerow(row)


    f = open('./data/{0}/intercepts_visible_{1}_{2}_{3}.csv'.format(projectName,file_name,times1,times2),'w')
    csvWriter = csv.writer(f)

    for count,metrics in enumerate(intercepts_visible):
        csvWriter.writerow("".format(count))
        csvWriter.writerow(metrics)


    f = open('./data/{0}/{1}_metrics_{2}_{3}.csv'.format(projectName,file_name,times1,times2),'w')
    csvWriter = csv.writer(f)

    for metrics in AE_feature_data_list:
        for metric in metrics:
            csvWriter.writerow(metric)

        csvWriter.writerow("")


cal_10_fold_cv(int(PARA[1]),int(PARA[2]))
