import numpy as np
import csv
import pyper
import sys
import da_tool.tca
import DCV
import gc
import math
import os
import error_bootstrap
sys.path.append("./../Util/")

import preparation
import clustering
import evaluation

PARA = sys.argv

csvName = ['tomcat_v6_0']
projectName = "PROMISE"


def data_extend(filename):
    train_data = []
    train_target = []

    f = open(filename,'r')
    dataReader = csv.reader(f)
    for row in dataReader:
        train_data.append(list(map(float,row[3:-1])))
        train_target.append(float(row[-1]))

    f.close()

    train_data = np.array(train_data)

    temp = []
    for i in train_target:
        if i != 0:
            temp.append(1)
        else:
            temp.append(0)

    train_target = np.array(temp)

    return [train_data, train_target]


def cal_10_fold_cv(times1,times2):

    file_name = csvName[int(PARA[3])]

    r = pyper.R()
    r("set.seed(200)")

    r('library("caret")')

    ALL = []

    LMT = [] # Logistic Model Tree
    J48 = [] # Decision Tree (J48)
    LR = [] # Logistic Regression
    NB = [] # Naive Bayes
    RF = [] # Random Forest

    feature_data_list = []
    preparation_error_list = []

    cnt = 1
    old_cnt = 1
    flag = True
    for times in range(times1,times2):
    #for times in range(1,2):


        print("+++ {0} times {1} fold +++".format(times,file_name))


        train_data = []
        train_target = []
        test_data = []
        test_target = []

        filename = "./../../data/{0}/{1}/train_times{2}.csv".format(projectName, file_name, times)
        train_data, train_target = data_extend(filename)

        filename = "./../../data/{0}/{1}/test_times{2}.csv".format(projectName, file_name, times)
        test_data, test_target = data_extend(filename)

        while(flag):
        
            rule = DCV.DCV(train_data, test_data)
            print(rule)
            if rule==1:
                train_data_std = train_data
                test_data_std = test_data
            elif rule==2:
                train_data_std, test_data_std = preparation.standardization_SVL(train_data,test_data,"0-1scale")
            elif rule==3:
                train_data_std, test_data_std = preparation.standardization_rule3(train_data,test_data)
            elif rule==4:
                train_data_std, test_data_std = preparation.standardization_rule4(train_data,test_data)
            else:
                train_data_std, test_data_std = preparation.standardization_SVL(train_data,test_data,"z-score")

            flag_in = 0
            for train_row in train_data_std:
                for ele in train_row:
                    if math.isnan(ele):
                        flag_in= 1
                        break
                if flag_in==1:
                    break
            if flag_in==1:
                error_bootstrap.new_bootstrap(file_name, projectName, times, cnt)

                filename = "./tmp/{0}_train_{1}_times{2}.csv".format(projectName, file_name, times)
                train_data, train_target = data_extend(filename)

                filename = "./tmp/{0}_test_{1}_times{2}.csv".format(projectName, file_name, times)
                test_data, test_target = data_extend(filename)

                cnt += 1
                print('train_cnt'.format(cnt))
                continue

            for test_row in test_data_std:
                for ele in test_row:
                    if math.isnan(ele):
                        flag_in = 1
                        break
                if flag_in==1:
                    break
            if flag_in==1:
                error_bootstrap.new_bootstrap(file_name, projectName, times, cnt)

                filename = "./tmp/{0}_train_{1}_times{2}.csv".format(projectName, file_name, times)
                train_data, train_target = data_extend(filename)

                filename = "./tmp/{0}_test_{1}_times{2}.csv".format(projectName, file_name, times)
                test_data, test_target = data_extend(filename)

                cnt += 1
                print('test_cnt'.format(cnt))
                continue

            break
        if cnt != old_cnt:
            print('Preparation error it contains nan')
            preparation_error_list.append(times)
            old_cnt = int(cnt)


        model = da_tool.tca.TCA(dim=10,kerneltype='rbf',kernelparam=1,mu=1)
        train_data_std, test_data_std, something = model.fit_transform(train_data_std, test_data_std)


        feature_data_list.append(test_data_std)


        #ans = cslLMTbyR(train_data_std)
        ans_LMT,ans_J48,ans_NB,ans_RF,ans_LR = clustering.Caret(train_data_std,train_target,test_data_std,test_target,r)


        #print('Conventional with LMT')
        temp = evaluation.calAUC_SVL(ans_LMT,test_target)
        #print(temp)
        LMT.append(temp)


        temp = evaluation.calAUC_SVL(ans_J48,test_target)
        #print(temp)
        J48.append(temp)


        temp = evaluation.calAUC_SVL(ans_NB,test_target)
        #print(temp)
        NB.append(temp)


        temp = evaluation.calAUC_SVL(ans_RF,test_target)
        #print(temp)
        RF.append(temp)


        temp = evaluation.calAUC_SVL(ans_LR,test_target)
        #print(temp)
        LR.append(temp)


        del train_data
        del train_target
        del train_data_std
        del test_data
        del test_target
        del test_data_std
        gc.collect()


    ALL.append(LMT)
    ALL.append(J48)
    ALL.append(LR)
    ALL.append(NB)
    ALL.append(RF)

    f = open('./data/{0}/error_preparation_error_{1}_{2}_{3}.csv'.format(projectName,file_name,times1,times2),'w')
    csvWriter = csv.writer(f)
    for row in preparation_error_list:
        csvWriter.writerow([row])


    f = open('./data/{0}/error_{1}_{2}_{3}.csv'.format(projectName,file_name,times1,times2),'w')
    csvWriter = csv.writer(f)

    for metrics in ALL:
        csvWriter.writerow(metrics)



    f = open('./data/{0}/error_{1}_metrics_{2}_{3}.csv'.format(projectName,file_name,times1,times2),'w')
    csvWriter = csv.writer(f)

    for metrics in feature_data_list:
        for metric in metrics:
            csvWriter.writerow(metric)

        csvWriter.writerow("")


cal_10_fold_cv(int(PARA[1]),int(PARA[2]))
