import csv
import numpy as np
import pandas as pd
from scipy.stats import iqr
import sys

#ProjectName = ['ant v1.7','camel v1.6','ivy v1.4','jedit v4.0','log4j v1.0','lucene v2.4','poi v3.0','tomcat v6.0','xalan v2.6','xerces v1.3','cm1','jm1','kc3','mc1','mc2','mw1','pc1','pc2','pc3','pc4','pc5','Eclipse JDT Core','Equinox','Apache Lucene','Mylyn','Eclipse PDE UI']

projectName = [['ant_v1_7','camel_v1_6','ivy_v1_4','jedit_v4_0','log4j_v1_0','lucene_v2_4','poi_v3_0','tomcat_v6_0','xalan_v2_6','xerces_v1_3'],['cm1','jm1','kc3','mc1','mc2','mw1','pc1','pc2','pc3','pc4','pc5'],['eclipse','equinox','lucene','mylyn','pde']]

DatasetName = ['PROMISE','NASA','AEEEM']

TECH = ['RBM','AutoEnco','PCA','FastMap','FeatureAgglomeration','RandomProjection','TCA','TCA+','PRE']

project_output = ['Ant v1.7','Camel v1.6','Ivy v1.4','Jedit v4.0','Log4j v1.0','Lucene v2.4','POI v3.0','Tomcat v6.0','Xalan v2.6','Xerces v1.3','CM1','JM1','KC3','MC1','MC2','MW1','PC1','PC2','PC3','PC4','PC5','Eclipse JDT Core','Equinox','Apache Lucene','Mylyn','Eclipse PDE UI','COUNT']

TECH_OUTPUT = ['RBM','AE','PCA','FM','FA','RP','TCA','TCA+','ORG']
#TECH_OUTPUT = ['RBM','AE','PCA','FM','FA','RP','PRE']

modelName = ['LMT', 'J48', 'LR', 'NB', 'RF']

def read_data():


    ALL = {}
    for d_name, _p_names in zip(DatasetName, projectName):
        ALL[d_name] = {}
        #df = pd.DataFrame()

        for p_name in _p_names:

            ALL[d_name][p_name] = {}

            for t_name in TECH:

                ALL[d_name][p_name][t_name] = []

                if t_name in ['RBM','PCA','PRE']:
                    f = open('./../{0}/data/{1}/{2}_1_100.csv'.format(t_name, d_name, p_name))
                else:
                    #if t_name=='TCA+' and p_name in ['tomcat_v6_0', 'lucene']:
                    #    f = open('./../{0}/data/{1}/error_{2}_1_101.csv'.format(t_name, d_name, p_name))
                    #else:
                    f = open('./../{0}/data/{1}/{2}_1_101.csv'.format(t_name, d_name, p_name))
                reader = f.readlines()

                for row in reader:
                    #print(row.split(','))
                    ALL[d_name][p_name][t_name].append(list(map(float,row.split(','))))
                    #print(np.median(np.array(list(map(float,row.split(','))))))

                #ALL[d_name][p_name][t_name] = round(iqr(ALL[d_name][p_name][t_name]),3)
                #f.close()

                
                #print(ALL[d_name][p_name][t_name])
                #print(np.median(np.array(ALL[d_name][p_name][t_name])))
                #sys.exit()
    return ALL



# ===============================
# x: the original data (list, int, array)
# samples: the number of extracted data
# ===============================
def bootstrap(data, samples):

    resampled_data = np.random.choice(data, samples)

    #print(resampled_data)

    return resampled_data

def median_values_with_bootstrap(data, repeats):

    medians = []
    for i in range(repeats):
        resampled_data = bootstrap(data, len(data))
        medians.append(np.median(resampled_data))

    return medians


def bootstrap_IQR(data_list):

    medians = []
    for data in data_list:

        medians.extend(median_values_with_bootstrap(data, 100))

    #print(len(medians))
    return round(iqr(medians), 3)

def make_color(row):
    temp = np.array(row)
    #index = np.argmin(temp)
    index = np.where(temp==temp.min())[0]

    new_row = []
    for n in row:
        new_row.append('%03.3f' % n)

    for i in index:
        new_row[i] = '\\cellcolor[gray]{0.85}{%03.3f}' % row[i]
    new_row[-1] = '{0}\\\\'.format(new_row[-1])

    return new_row, index

def write_tex_table(ALL):
    TEX_TABLE = []
    TABLE = []
    COUNTER = np.array(np.zeros(len(TECH)))
    for d_name, _p_names in zip(DatasetName, projectName):
        for p_name in _p_names:
            row = []
            for t_name in TECH:

                #row.append(ALL[d_name][p_name][t_name])
                #print(len(ALL[d_name][p_name][t_name]))
                IQR = bootstrap_IQR(ALL[d_name][p_name][t_name])
                row.append(IQR)

            TABLE.append(row)
            row, index = make_color(row)
            COUNTER[index] += 1
            TEX_TABLE.append(row)

    TEX_TABLE.append(list(map(int,list(COUNTER))))
    TEX_TABLE = pd.DataFrame(TEX_TABLE, columns=TECH_OUTPUT, index=project_output)
    TEX_TABLE.to_csv(path_or_buf='./tables/SVL.csv', sep='&')

    f = open('./delta/SVL_SK_ESD_data.csv','w')
    csvWriter = csv.writer(f)

    csvWriter.writerow(TECH_OUTPUT)
    for row in TABLE:
        csvWriter.writerow(row)


    

np.random.seed(200)
ALL =  read_data()
write_tex_table(ALL)
#write_data(ALL)

#a = [1,2,3,4,5,6,7,8,9,10,0.3,0.2,0.9]
#median_values_with_bootstrap(a, 10)

#a = [1,2,3,4,5,6,7,8,9,10,0.3,0.2,0.9]
#print(bootstrap(a, 10))
#print(bootstrap(a, 10))


