#-*- coding: utf-8 -*-
import numpy as np
import csv
import pyper
import sys
import gc
sys.path.append("./../Util/")

import preparation
import clustering
import evaluation
import FS

PARA = sys.argv


csvName = ['cm1','jm1','kc3','mc1','mc2','mw1','pc1','pc2','pc3','pc4','pc5']
projectName = 'NASA'


def data_extend(filename):
    train_data = []
    train_target = []

    f = open(filename,'r')
    dataReader = csv.reader(f)
    for row in dataReader:
        train_data.append(list(map(float,row[:-1])))
        train_target.append(row[-1])

    f.close()

    train_data = np.array(train_data)

    temp = []
    for i in train_target:
        if i == 'Y' or i=='TRUE':
            temp.append(1)
        elif i=='N' or i=='FALSE':
            temp.append(0)
        else:
            print('error')

    train_target = temp

    train_target = np.array(temp)

    return [train_data, train_target]


def cal_10_fold_CV_PROMISE(csvName,projectName):
    ALL = {}

    r = pyper.R()
    r("set.seed(200)")
    r('library("FSelector")')

    ALL['Error of SC'] = []
    ALL['Error of FCM'] = []
    ALL['Error of test_data'] = []

    for file_name in csvName:
        ALL[file_name] = []
        count = 0
        all_count = 0
        count_FCM = 0

        ConFS_feature_data_list = []

        SC = []
        PAM = []
        NG = []
        FCM = []
        KM = []
        error_list = []

        for times in range(int(PARA[1]),int(PARA[2])+1):

            print("+++ {0} times {1} fold +++".format(times,file_name))

            train_data = []
            train_target = []
            test_data = []
            test_target = []

            filename = "./../../data/{0}/{1}/train_times{2}.csv".format(projectName, file_name, times)
            train_data, train_target = data_extend(filename)

            filename = "./../../data/{0}/{1}/test_times{2}.csv".format(projectName, file_name, times)
            test_data, test_target = data_extend(filename)

            if sum(train_target)==0 or sum(train_target)==len(train_target):
                all_count = all_count + 1
                print("---train data error---")
                sys.exit()

            if sum(test_target)==0 or sum(test_target)==len(test_target):
                all_count = all_count + 1
                print("---test data error---")
                continue


            train_data_std, test_data_std = preparation.standardization_SVL(train_data,test_data,"z-score")

            train_data_std, test_data_std = FS.ConFS(train_data_std, train_target, test_data_std, r)

            ConFS_feature_data = test_data_std

            ConFS_feature_data_list.append(ConFS_feature_data)

            if len(ConFS_feature_data[0])==1:
                double_ConFS_feature_data = np.c_[ConFS_feature_data,ConFS_feature_data]
            else:
                double_ConFS_feature_data = ConFS_feature_data

            #ans = cslSCbyR(test_data_std)
            ans = clustering.cslSCbyR(ConFS_feature_data,RBM=True)

            if ans != 1:

                #print('ConFS with SC')
                temp = evaluation.calAUC(ans,test_target,test_data_std)
                #print(temp)
                SC.append(temp)
            else :
                print("Error")
                error_list.append([times])


            #ConFS_feature_data = model.fit_transform(test_data_std)

            ans = clustering.cslPAMbyR(ConFS_feature_data)
            #ans = clustering.cslPAMbyR(test_data)

            #print('ConFS with PAM')
            temp = evaluation.calAUC(ans,test_target,test_data_std)
            #print(temp)
            PAM.append(temp)


            ans = clustering.cslNGbyR(double_ConFS_feature_data)

            #print('ConFS with NG')
            temp = evaluation.calAUC(ans,test_target,test_data_std)
            #print(temp)
            NG.append(temp)


            #ans = cslSCbyR(test_data_std)
            ans = clustering.cslFCMbyR(ConFS_feature_data)

            #print('ConFS with FCM')
            temp = evaluation.calAUC(ans,test_target,test_data_std)
            #print(temp)
            FCM.append(temp)


            ans = clustering.cslKMbyR(double_ConFS_feature_data)

            #print('ConFS with KM')
            temp = evaluation.calAUC(ans,test_target,test_data_std)
            #print(temp)
            KM.append(temp)

            del train_data
            del train_target
            del train_data_std
            del test_data
            del test_target
            del test_data_std
            del ConFS_feature_data
            gc.collect()

        ALL[file_name].append(SC)
        ALL[file_name].append(PAM)
        ALL[file_name].append(NG)
        ALL[file_name].append(FCM)
        ALL[file_name].append(KM)
        ALL['Error of SC'].append(count)
        ALL['Error of test_data'].append(all_count)

        print(" --- result of {0} average --- ".format(file_name))
        print("SC : {0}".format(sum(ALL[file_name][0])/len(ALL[file_name][0])))
        print("PAM : {0}".format(sum(ALL[file_name][1])/len(ALL[file_name][1])))
        print("NG : {0}".format(sum(ALL[file_name][2])/len(ALL[file_name][2])))
        print("FCM : {0}".format(sum(ALL[file_name][3])/len(ALL[file_name][3])))
        print("KM : {0}".format(sum(ALL[file_name][4])/len(ALL[file_name][4])))


        f = open('./data/{0}/SC_Error_{1}_{2}_{3}.csv'.format(projectName,file_name,PARA[1],PARA[2]),'w')
        csvWriter = csv.writer(f)
        for row in error_list:
            csvWriter.writerow([row])


        f = open('./data/{1}/{0}_{2}_{3}.csv'.format(file_name,projectName,PARA[1],PARA[2]),'w')
        csvWriter = csv.writer(f)

        for metrics in ALL[file_name]:
            csvWriter.writerow(metrics)


        f = open('./data/{1}/{0}_metrics_{2}_{3}.csv'.format(file_name,projectName,PARA[1],PARA[2]),'w')
        csvWriter = csv.writer(f)

        for metrics in ConFS_feature_data_list:
            for metric in metrics:
                csvWriter.writerow(metric)

            csvWriter.writerow("")

cal_10_fold_CV_PROMISE(csvName,projectName)
