import csv
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from collections import OrderedDict

projectName = {"PROMISE":['ant v1.7','camel v1.6','ivy v1.4','jedit v4.0','log4j v1.0','lucene v2.4','poi v3.0','tomcat v6.0','xalan v2.6','xerces v1.3'],"NASA":['cm1','jm1','kc3','mc1','mc2','mw1','pc1','pc2','pc3','pc4','pc5'],"AEEEM":['Eclipse JDT Core','Equinox','Apache Lucene','Mylyn','Eclipse PDE UI']}
datasetName = ['PROMISE','NASA','AEEEM']
project_output = {"PROMISE":['Ant v1.7','Camel v1.6','Ivy v1.4','Jedit v4.0','Log4j v1.0','Lucene v2.4','POI v3.0','Tomcat v6.0','Xalan v2.6','Xerces v1.3'],"NASA":['CM1','JM1','KC3','MC1','MC2','MW1','PC1','PC2','PC3','PC4','PC5'],"AEEEM":['Eclipse JDT Core','Equinox','Apache Lucene','Mylyn','Eclipse PDE UI']}

#projectName = {"PROMISE":['ant v1.7']}
#datasetName = ['PROMISE']


modelName = ['LMT','J48','LR','NB','RF']

TECH = ['RBM','AE','PCA','FM','FA','RP','TCA','TCA+']


def initialize_dict():

    dic = OrderedDict()
    for d_name in datasetName:
        dic[d_name] = OrderedDict()
        for p_name in projectName[d_name]:
            dic[d_name][p_name] = OrderedDict()
            for t_name in TECH:
                dic[d_name][p_name][t_name] = OrderedDict()
                for m_name in modelName:
                    dic[d_name][p_name][t_name][m_name] = 0
    return dic


def read_data():

    result_dic = initialize_dict()

    for d_name in datasetName:

        for p_name in projectName[d_name]:

            f = open('./performance_data/RQ1_SVL_{0}_{1}.csv'.format(d_name,p_name),'r')
            reader = csv.reader(f)

            head = next(reader)
            data = []

            for row in reader:
                data.append(list(map(float, row)))

            data = np.array(data)
            ORG = data[:,-1]
            data = data.T

            ratio_data = data/ORG

            for cnt, t_name in enumerate(TECH):
                for m_name, num in zip(modelName, ratio_data[cnt]):
                    result_dic[d_name][p_name][t_name][m_name] = num

    #print(result_dic)

    return result_dic


def make_table(result_dic):

    output = []
    for d_name in datasetName:
        output.extend(project_output[d_name])

    for m_name in modelName:
        TABLE = []
        for d_name in datasetName:
            for p_name in projectName[d_name]:
                row = []
                for t_name in TECH:
                    if result_dic[d_name][p_name][t_name][m_name] >= 1:
                        row.append('%03.3f' % round(result_dic[d_name][p_name][t_name][m_name],3))
                        row[-1] = '\\cellcolor[gray]{{0.85}}{{{0}}}'.format(row[-1])
                    else:
                        row.append('%03.3f' % round(result_dic[d_name][p_name][t_name][m_name],3))

                row[-1] = '{0}\\\\'.format(row[-1])
                TABLE.append(row)

        TABLE = pd.DataFrame(TABLE,columns=TECH,index=output)
        TABLE.to_csv(path_or_buf='./tables/{0}.csv'.format(m_name),sep="&")


def plot(data,name,d_name):

    #plt.figure(figsize=(14,6))
    plt.figure()
    plt.boxplot(data)

    plt.xticks([i+1 for i in range(len(project_output[d_name]))],project_output[d_name],rotation=45)
    plt.hlines(y=1, xmin=0, xmax=28, colors='b', linewidth=1, linestyles='dashed')

    plt.legend()
    plt.tight_layout()

    plt.savefig("./performance_plot/" + name + ".pdf")
    #plt.savefig("./plot/" + name + ".png")

def plot_subfigure(data,name):

    plt.figure()
    fig, ax = plt.subplots(nrows=1,ncols=3,figsize=(16,6),sharey=True)
    fs = 20

    ax[0].boxplot(data[0],labels=project_output['PROMISE'])
    for tick in ax[0].get_xticklabels():
        tick.set_rotation(45)
    ax[0].hlines(y=1, xmin=0, xmax=28, colors='b', linewidth=1, linestyles='dashed')
    ax[0].set_title('PROMISE', fontsize=fs)
    ax[0].tick_params(labelsize=15)

    ax[1].boxplot(data[1],labels=project_output['NASA'])
    for tick in ax[1].get_xticklabels():
        tick.set_rotation(45)
    ax[1].hlines(y=1, xmin=0, xmax=28, colors='b', linewidth=1, linestyles='dashed')
    ax[1].set_title('NASA', fontsize=fs)
    ax[1].tick_params(labelsize=15)

    ax[2].boxplot(data[2],labels=project_output['AEEEM'])
    for tick in ax[2].get_xticklabels():
        tick.set_rotation(45)
    ax[2].hlines(y=1, xmin=0, xmax=28, colors='b', linewidth=1, linestyles='dashed')
    ax[2].set_title('AEEEM', fontsize=fs)
    ax[2].tick_params(labelsize=15)


    #plt.tick_params(labelsize=15)
    #plt.rcParams.update({'axes.labelsize':'large'})
    plt.legend()
    plt.tight_layout()

    plt.savefig("./performance_plot/" + name + ".pdf")
    #plt.savefig("./plot/" + name + ".png")




def boxplot(result_dic):

    data = []
    for d_name in datasetName:
        temp = []
        for p_name in projectName[d_name]:
            row = []
            for m_name in modelName:
                for t_name in TECH:
                    row.append(result_dic[d_name][p_name][t_name][m_name])

            temp.append(row)
        data.append(temp)
        plot(temp,'SVL_{0}'.format(d_name),d_name)

    plot_subfigure(data,'SVL')


if __name__=='__main__':

    result_dic = read_data()
    #make_table(result_dic)
    boxplot(result_dic)




