#!/bin/sh
R --vanilla --slave << 'EOM'

library(ScottKnottESD)

library(reshape2)
library(ggplot2)

# Create legend vector.
make_legend <- function(sk,dat){
    v = as.vector(NULL)
    for(nms in dat$variable){
        if(nms=='USVL_TCA+'){
            temp = sk$groups['USVL_TCA.']
        }
        else if(nms=='SVL_TCA+'){
            temp = sk$groups['SVL_TCA.']
        }
        else{
            temp = sk$groups[nms]
        }
        v = c(v,temp)
    }
    return(v)
}


print("USVL SK test")
dat = read.csv("./performance_data/RQ3_data_ESD.csv",header=T,check.names=FALSE)
sk <- sk_esd(dat)

dat = melt(dat)
dat$variable <- with(dat, reorder(variable,-value,mean))

V2 = make_legend(sk,dat)
dat <- transform(dat,V2=V2)

g <- ggplot(
    dat,
    aes(
        x = variable,
        y = value,
        fill=factor(V2)
    )
)
g <- g + geom_boxplot(aes(linetype=factor(V2)))
g <- g + ylab("AUC")
g <- g + xlab("")
g <- g + coord_cartesian(ylim=c(0.45,0.9))
g <- g + theme(axis.text.x = element_text(angle = 45, hjust = 1, size=20), axis.text.y = element_text(size=20))
g <- g + theme(axis.title.y = element_text(size=20))
#g <- g + scale_fill_discrete(name="Legend",labels=c('Group 1','Group 2','Group 3','Group 4'))
#g <- g + scale_linetype_discrete(name="Legend",labels=c('Group 1','Group 2','Group 3','Group 4'))
g <- g + theme(legend.position="none")
g <- g + theme(panel.grid.major=element_blank(), panel.grid.minor=element_blank(), panel.background=element_blank())
g <- g + theme(axis.line.x = element_line(color="black", size = 0.5),axis.line.y = element_line(color="black", size = 0.5))

pdf("./performance_plot/RQ3_SK_ESD.pdf",width=14)
plot(g)


EOM
