import csv
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from collections import OrderedDict
from statistics import median
import sys

projectName = {"PROMISE":['ant v1.7','camel v1.6','ivy v1.4','jedit v4.0','log4j v1.0','lucene v2.4','poi v3.0','tomcat v6.0','xalan v2.6','xerces v1.3'],"NASA":['cm1','jm1','kc3','mc1','mc2','mw1','pc1','pc2','pc3','pc4','pc5'],"AEEEM":['Eclipse JDT Core','Equinox','Apache Lucene','Mylyn','Eclipse PDE UI']}
datasetName = ['PROMISE','NASA','AEEEM']

#projectName = {"PROMISE":['ant v1.7']}
#datasetName = ['PROMISE']

#projectName = {"PROMISE":['ant v1.7','camel v1.6','ivy v1.4','jedit v4.0','log4j v1.0','lucene v2.4','poi v3.0','tomcat v6.0','xalan v2.6','xerces v1.3']}
#datasetName = ['PROMISE']

project_output = {"PROMISE":['Tech','Ant v1.7','Camel v1.6','Ivy v1.4','Jedit v4.0','Log4j v1.0','Lucene v2.4','POI v3.0','Tomcat v6.0','Xalan v2.6','Xerces v1.3','COUNT'],"NASA":['Tech','CM1','JM1','KC3','MC1','MC2','MW1','PC1','PC2','PC3','PC4','PC5','COUNT'],"AEEEM":['Tech','Eclipse JDT Core','Equinox','Apache Lucene','Mylyn','Eclipse PDE UI','COUNT']}

modelName = {'SVL':['LMT','J48','LR','NB','RF'], 'USVL':['SC','PAM','NG','FCM','KM']}
#modelName_output = ['LMT','J48','LR','NB','RF','SC','PAM','NG','FCM','KM']

modelType = ['SVL','USVL']
#modelType = ['USVL']

INDEX = ['','SVL','','','','USVL','','']

TECH = ['RBM','AE','PCA','FM','FA','RP','TCA','TCA+']

best_techs = {'SVL': ['FA','PCA','TCA'], 'USVL': ['RBM','AE','FA','PCA','TCA']}
#best_techs = {'USVL': ['RBM']}
def initialize_dict():

    dic = OrderedDict()
    for d_name in datasetName:
        dic[d_name] = OrderedDict()
        for p_name in projectName[d_name]:
            dic[d_name][p_name] = OrderedDict()
            for t_name in TECH:
                dic[d_name][p_name][t_name] = OrderedDict()
                for m_name in modelName:
                    dic[d_name][p_name][t_name][m_name] = 0
    return dic


def read_data_SVL():

    result_dic = initialize_dict()

    for d_name in datasetName:

        for p_name in projectName[d_name]:

            f = open('./../Figures/performance_data/RQ1_SVL_{0}_{1}.csv'.format(d_name, p_name),'r')
            reader = csv.reader(f)

            head = next(reader)
            data = []

            for row in reader:
                data.append(list(map(float, row)))

            data = np.array(data)
            ORG = data[:,-1]
            data = data.T

            ratio_data = data/ORG

            for cnt, t_name in enumerate(TECH):
                for m_name, num in zip(modelName['SVL'], ratio_data[cnt]):
                    result_dic[d_name][p_name][t_name][m_name] = num

    #print(result_dic)

    return result_dic

def read_data_USVL():

    result_dic = initialize_dict()

    for d_name in datasetName:

        for p_name in projectName[d_name]:

            f = open('./../Figures/performance_data/RQ1_USVL_{0}_{1}.csv'.format(d_name, p_name),'r')
            reader = csv.reader(f)

            head = next(reader)
            data = []

            for row in reader:
                data.append(list(map(float, row)))

            data = np.array(data)
            ORG = data[:,-1]
            data = data.T

            ratio_data = data/ORG

            for cnt, t_name in enumerate(TECH):
                for m_name, num in zip(modelName['USVL'], ratio_data[cnt]):
                    result_dic[d_name][p_name][t_name][m_name] = num

    #print(result_dic)

    return result_dic


def make_table(result_dic):

    for d_name in datasetName:
        TABLE = []
        for m_type in modelType:
            for t_name in best_techs[m_type]:
                data = [t_name]
                cnt = 0
                for p_name in projectName[d_name]:
                    row = []
                    for m_name in modelName[m_type]:
                        row.append(result_dic[m_type][d_name][p_name][t_name][m_name])

                    med = median(row)
                    if round(med,3) > 1.0:
                        data.append('\\cellcolor[gray]{0.85}{%03.3f}' % (round(med,3)))
                        cnt += 1
                    else:
                        data.append('%03.3f' % (round(med,3)))
                data.append('{0}\\\\'.format(cnt))
                TABLE.append(data)

        TABLE = pd.DataFrame(TABLE,columns=project_output[d_name],index=INDEX)
        TABLE.to_csv(path_or_buf='./tables/RQ1_{0}.csv'.format(d_name),sep="&")


if __name__=='__main__':

    result_dic = {}

    result_dic['SVL'] = read_data_SVL()
    result_dic['USVL'] = read_data_USVL()

    #boxplot_projects(result_dic)
    #boxplot_models(result_dic)

    make_table(result_dic)


