import csv
import numpy as np
import pandas as pd
from scipy.stats import iqr

ProjectName = ['ant v1.7','camel v1.6','ivy v1.4','jedit v4.0','log4j v1.0','lucene v2.4','poi v3.0','tomcat v6.0','xalan v2.6','xerces v1.3','cm1','jm1','kc3','mc1','mc2','mw1','pc1','pc2','pc3','pc4','pc5','Eclipse JDT Core','Equinox','Apache Lucene','Mylyn','Eclipse PDE UI']

DatasetName = ['PROMISE','NASA','AEEEM']

Write_Name = ["RQ3_table_PROMISE.csv","RQ3_table_NASA.csv","RQ3_table_AEEEM.csv"]

TECH = ['RBM','AE','PCA','FM','FA','RP','TCA','TCA+']
#TECH = ['RBM','AE','PCA','FM','FA','RP']
NUM = [2,1,5,3,4,6,7,8]

TECH_OUTPUT = ['RBM','AE','PCA','FM','FA','RP','TCA','TCA+','ORG']
#TECH_OUTPUT = ['RBM','AE','PCA','FM','FA','RP','PRE']

project_output = ['Ant v1.7','Camel v1.6','Ivy v1.4','Jedit v4.0','Log4j v1.0','Lucene v2.4','POI v3.0','Tomcat v6.0','Xalan v2.6','Xerces v1.3','CM1','JM1','KC3','MC1','MC2','MW1','PC1','PC2','PC3','PC4','PC5','Eclipse JDT Core','Equinox','Apache Lucene','Mylyn','Eclipse PDE UI']

def newlines(data):

    IQR = iqr(data)
    Q1 = np.percentile(data,25)
    Q3 = np.percentile(data,75)
    upper_bound = Q3 + IQR*1.5
    lower_bound = Q1 - IQR*1.5

    print('A')
    print(upper_bound)
    print(lower_bound)
    new_data = []
    for num in data:
        if num > upper_bound or num < lower_bound:
            new_data.append('\\cellcolor[gray]{0.85}{%03.3f}' % (num))
        else:
            new_data.append('%03.3f' % (num))

    #new_data[-1] = '{0}\\\\'.format(new_data[-1])

    return new_data

def add_newlines(data):
    
    data[-1] = '{0}\\\\'.format(data[-1])

    return data

def read_data():

    LMT = {}
    J48 = {}
    LR = {}
    NB = {}
    RF = {}
    conventional_LMT = []
    conventional_J48 = []
    conventional_LR = []
    conventional_NB = []
    conventional_RF = []

    for d_name in DatasetName:
        LMT['{0}'.format(d_name)] = {}
        J48['{0}'.format(d_name)] = {}
        LR['{0}'.format(d_name)] = {}
        NB['{0}'.format(d_name)] = {}
        RF['{0}'.format(d_name)] = {}
        for num, tech in zip(NUM,TECH):

            f = open('./../FR_SVL/test/RQ{0}/{1}_median_{2}_Super.txt'.format(num,tech,d_name))
            reader = f.readlines()

            LMT['{0}'.format(d_name)]['{0}'.format(tech)] = []
            J48['{0}'.format(d_name)]['{0}'.format(tech)] = []
            LR['{0}'.format(d_name)]['{0}'.format(tech)] = []
            NB['{0}'.format(d_name)]['{0}'.format(tech)] = []
            RF['{0}'.format(d_name)]['{0}'.format(tech)] = []


            for row in reader:
                text = row.replace('\n','').split("  ")
                #print(text)

                if text[0]=='{0}_LMT'.format(tech):
                    LMT['{0}'.format(d_name)]['{0}'.format(tech)].append(str(round(float(text[1]),3)))
                elif text[0]=='{0}_J48'.format(tech):
                    J48['{0}'.format(d_name)]['{0}'.format(tech)].append(str(round(float(text[1]),3)))
                elif text[0]=='{0}_LR'.format(tech):
                    LR['{0}'.format(d_name)]['{0}'.format(tech)].append(str(round(float(text[1]),3)))
                elif text[0]=='{0}_NB'.format(tech):
                    NB['{0}'.format(d_name)]['{0}'.format(tech)].append(str(round(float(text[1]),3)))
                elif text[0]=='{0}_RF'.format(tech):
                    RF['{0}'.format(d_name)]['{0}'.format(tech)].append(str(round(float(text[1]),3)))
                else:
                    pass

            f.close()


        f = open('./../FR_SVL/test/RQ1/PRE_median_{0}_Super.txt'.format(d_name),'r')
        reader = f.readlines()


        for row in reader:
            text = row.replace('\n','').split("  ")
            #print(text)

            if text[0]=='PRE_LMT':
                conventional_LMT.append(text[1])
            elif text[0]=='PRE_J48':
                conventional_J48.append(text[1])
            elif text[0]=='PRE_LR':
                conventional_LR.append(text[1])
            elif text[0]=='PRE_NB':
                conventional_NB.append(text[1])
            elif text[0]=='PRE_RF':
                conventional_RF.append(text[1])
            else:
                pass

        f.close()


    TABLE = []
    for tech in TECH:
        row = []

        for d_name in DatasetName:

            for a,b,c,d,e in zip(LMT[d_name][tech],J48[d_name][tech],LR[d_name][tech],NB[d_name][tech],RF[d_name][tech]):

                temp = [a,b,c,d,e]
                temp = list(map(float,temp))

                #row.append('%03.3f' % (round(iqr(temp),3)))
                row.append(round(iqr(temp),3))

        TABLE.append(row)

    row = []
    for a,b,c,d,e in zip(conventional_LMT,conventional_J48,conventional_LR,conventional_NB,conventional_RF):

        temp = [a,b,c,d,e]
        temp = list(map(float,temp))
        #row.append('%03.3f' % (round(iqr(temp),3)))
        row.append(round(iqr(temp),3))

    TABLE.append(row)
    #print(TABLE)
    TABLE = list(map(newlines, TABLE))

    TABLE = np.array(TABLE).T
    #TABLE = list(map(list,TABLE))

    TABLE = list(map(add_newlines, TABLE))

    TABLE = pd.DataFrame(TABLE,columns=TECH_OUTPUT,index=project_output)
    TABLE.to_csv(path_or_buf='./tables/RQ2_variance_SVL.csv',sep="&")


read_data()




